"""
    SleekXMPPGCM: XMPP Client for Google/Firebase Cloud Messaaging
    Copyright (C) 2016  Winster Jose
    This file is part of XMPPGCM.

    See the file LICENSE for copying permission.
"""


from .gcm import GCM, XMPPEvent